-- Main laod file for Real Time Sync Mod

local debug        = false
local modDirectory = g_currentModDirectory or ""
local modName      = g_currentModName or "unknown"
local modEnvironment

source(g_currentModDirectory .. 'scripts/RealTimeSync.lua')
source(g_currentModDirectory .. 'scripts/lib/fs22Logger.lua')
source(g_currentModDirectory .. 'scripts/lib/fs22SimpleUtils.lua')
source(g_currentModDirectory .. 'scripts/SettingEvent.lua')

local function load(mission)
	assert(g_realTimeSync == nil)

	local siLoggeer = FS22Log:new(
		"realTimeSync",
		debug and FS22Log.DEBUG_MODE.VERBOSE or FS22Log.DEBUG_MODE.WARNINGS,
		{
			"getValue",
			"setValue",
			"display_data",
			"outputTextLines"
		}
	)

	modEnvironment = RealTimeSync:new(mission, modDirectory, modName, siLoggeer)

	getfenv(0)["g_realTimeSync"] = modEnvironment

  if mission:getIsClient() then
		addModEventListener(modEnvironment)
  end
end

local function unload()
	removeModEventListener(modEnvironment)
	modEnvironment:delete()
	modEnvironment = nil -- Allows garbage collecting
	getfenv(0)["g_realTimeSync"] = nil
end

local function startMission(mission)
	modEnvironment:onStartMission(mission)
end

local function save()
	modEnvironment:save()
end

local function init()
	FSBaseMission.delete = Utils.appendedFunction(FSBaseMission.delete, unload)

	Mission00.load           = Utils.prependedFunction(Mission00.load, load)
	Mission00.onStartMission = Utils.appendedFunction(Mission00.onStartMission, startMission)

	InGameMenuGameSettingsFrame.onFrameOpen = Utils.appendedFunction(InGameMenuGameSettingsFrame.onFrameOpen, RealTimeSync.initGui)

	FSCareerMissionInfo.saveToXMLFile = Utils.appendedFunction(FSCareerMissionInfo.saveToXMLFile, save)
  -- Disables Sleeping in Game
  SleepManager.getCanSleep = Utils.overwrittenFunction(SleepManager.getCanSleep, RealTimeSync.disableSleep)
  -- Loads when user joins server
  FSBaseMission.onConnectionFinishedLoading = Utils.appendedFunction(FSBaseMission.onConnectionFinishedLoading,RealTimeSync.join)

end

init()